package cn.fw.valhalla.rpc.angel;

import cn.fw.angel.sdk.api.CustomerApiService;
import cn.fw.angel.sdk.api.InsuranceEstimateApiService;
import cn.fw.angel.sdk.api.InsurerApiSerivce;
import cn.fw.angel.sdk.result.CustomerInsuranceDTO;
import cn.fw.angel.sdk.result.EffectivePolicyDTO;
import cn.fw.angel.sdk.result.InsurerInfoDTO;
import cn.fw.angel.sdk.result.Message;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.rpc.angel.dto.InsuranceDTO;
import cn.fw.valhalla.rpc.angel.dto.InsurerDTO;
import com.alibaba.fastjson.JSONObject;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundValueOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * create at 2020-07-22
 *
 * @author kurisu
 */
@Slf4j
@Service
public class InsurerRpcService {

    private final InsurerApiSerivce insurerApiSerivce;
    private final CustomerApiService insCustomerApiService;
    private final InsuranceEstimateApiService insuranceEstimateApiService;
    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:rpc:angel")
    @Getter
    private String keyPrefix;

    @Autowired
    InsurerRpcService(final InsurerApiSerivce insurerApiSerivce,
                      final CustomerApiService insCustomerApiService,
                      final InsuranceEstimateApiService insuranceEstimateApiService,
                      final StringRedisTemplate redisTemplate) {
        this.insurerApiSerivce = insurerApiSerivce;
        this.insCustomerApiService = insCustomerApiService;
        this.insuranceEstimateApiService = insuranceEstimateApiService;
        this.redisTemplate = redisTemplate;
    }


    public InsurerDTO getById(Long insurerId) {
        try {
            Message<InsurerInfoDTO> msg = insurerApiSerivce.getById(insurerId);
            log.info("查询保险公司结果:{}", msg);
            if (!msg.isSuccess() || Objects.isNull(msg.getData())) {
                return null;
            }

            InsurerDTO dto = new InsurerDTO();
            BeanUtils.copyProperties(msg.getData(), dto);
            return dto;
        } catch (Exception e) {
            log.error("查询保险公司失败", e);
            return null;
        }
    }


    /**
     * 查询保险信息
     *
     * @param custId
     * @param insurerId
     * @return
     */
    public Optional<InsuranceDTO> getByInsId(Long custId, String insurerId) {
        try {
            if (custId == null || !StringUtils.isNumber(insurerId)) {
                return Optional.empty();
            }
            Message<EffectivePolicyDTO> msg = insuranceEstimateApiService.getEffectivePolicy(custId, Long.valueOf(insurerId));
            if (!msg.isSuccess() || Objects.isNull(msg.getData())) {
                log.error("查询保险信息失败");
                return Optional.empty();
            }
            InsuranceDTO dto = new InsuranceDTO();
            EffectivePolicyDTO data = msg.getData();
            log.info("查询保险信息: {}", JSONObject.toJSONString(data));
            BeanUtils.copyProperties(data, dto);
            return Optional.of(dto);
        } catch (Exception e) {
            log.error("查询保险公司失败", e);
            return Optional.empty();
        }
    }

    /**
     * 根据档案id查询正在生效的保险信息
     *
     * @param customerId
     * @return
     */
    public InsuranceDTO queryInsByCustId(Long customerId) {
        final String key = generateKey(customerId);
        try {
            String cache = getFromCache(key);
            if (StringUtils.isValid(cache)) {
                return JSONObject.parseObject(cache, InsuranceDTO.class);
            }
            Message<CustomerInsuranceDTO> msg = insCustomerApiService.getCustomerInsurance(customerId);
            log.info("查询正在生效的保险信息:{}", msg.getResult());
            CustomerInsuranceDTO data = msg.getData();
            if (!msg.isSuccess() || Objects.isNull(data)) {
                return null;
            }
            InsuranceDTO dto = new InsuranceDTO();
            BeanUtils.copyProperties(data, dto);
            setToCache(key, JSONObject.toJSONString(dto), 30);
            return dto;
        } catch (Exception e) {
            log.error("查询正在生效的保险信息失败：", e);
            return null;
        }
    }

    private String getFromCache(final String key) {
        String json = null;
        try {
            BoundValueOperations<String, String> ops = redisTemplate.boundValueOps(key);
            json = ops.get();
        } catch (Exception e) {
            log.error("从缓存获取保有客保险信息失败", e);
        }
        return json;
    }

    private void setToCache(final String key, final String value, long timeout) {
        try {
            redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("缓存保有客保险信息失败", e);
        }
    }


    private String generateKey(final Long customerId) {
        Assert.notNull(customerId, "customerId cannot be null");

        return String.format("%s:%s", getKeyPrefix(), customerId);
    }
}
