package cn.fw.valhalla.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.query.CustomerCluePoolQueryVO;
import cn.fw.valhalla.domain.query.FollowPoolQueryVO;
import cn.fw.valhalla.domain.query.StammkundePoolQueryVO;
import cn.fw.valhalla.domain.vo.follow.FollowDetailVO;
import cn.fw.valhalla.domain.vo.pool.*;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.bus.follow.PoolBizService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.valhalla.common.constant.MessageStr.QUERY_FAILURE;

/**
 * @author : kurisu
 * @className : StammkundePoolController
 * @description : 保有客池
 * @date: 2020-11-11 17:26
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@RequiredArgsConstructor
@Validated
@RequestMapping("/app/pool")
public class PoolController {

    private final PoolBizService poolBizService;
    private final FollowBizService followBizService;


    @GetMapping("/follow/own/list")
    public Message<AppPage<FollowPoolListVO>> ownFollowList(@CurrentUser LoginAuthBean currentUser, final FollowPoolQueryVO queryVO) {
        final String msg = "查询自己的跟进池列表[pool/own/follow/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            AppPage<FollowPoolListVO> page = poolBizService.followList(currentUser, queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO.getType(), e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/follow/list")
    @IgnoreAuth
    public Message<AppPage<FollowPoolListVO>> followList(@CurrentUser LoginAuthBean currentUser, final FollowPoolQueryVO queryVO) {
        final String msg = "查询跟进池列表[pool/follow/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            queryVO.setGroupId(currentUser.getGroupId());
            AppPage<FollowPoolListVO> page = poolBizService.followList(null, queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO.getType(), e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/follow/detail")
    @IgnoreAuth
    public Message<FollowDetailVO> detail(@NotNull(message = "跟进池id不能为空") final Long taskId) {
        final String msg = "查询跟进池详情[pool/follow/detail]";
        try {
            log.info("{}: param[{}]", msg, taskId);
            FollowDetailVO detailVO = followBizService.followPoolDetail(taskId);
            return success(detailVO, data -> log.info("{}", data));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, taskId, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/public/list")
    @IgnoreAuth
    public Message<AppPage<PublicPoolVO>> publicList(@CurrentUser LoginAuthBean currentUser, final FollowPoolQueryVO queryVO) {
        final String msg = "查询公共池列表[pool/public/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            queryVO.setGroupId(currentUser.getGroupId());
            AppPage<PublicPoolVO> page = poolBizService.publicList(currentUser, queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/stammkunde/own/list")
    public Message<AppPage<StammkundePoolVO>> ownStammkundeList(@CurrentUser LoginAuthBean currentUser, final StammkundePoolQueryVO queryVO) {
        final String msg = "查询自己的保有客池列表[pool/stammkunde/own/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            AppPage<StammkundePoolVO> page = poolBizService.stammkundeList(currentUser, queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/stammkunde/list")
    @IgnoreAuth
    public Message<AppPage<StammkundePoolVO>> stammkundeList(@CurrentUser LoginAuthBean currentUser, final StammkundePoolQueryVO queryVO) {
        final String msg = "查询保有客池列表[pool/stammkunde/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            queryVO.setGroupId(currentUser.getGroupId());
            AppPage<StammkundePoolVO> page = poolBizService.stammkundeList(null, queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/clue/list")
    @IgnoreAuth
    public Message<AppPage<CustomerCluePoolVO>> clueList(@CurrentUser LoginAuthBean currentUser, final CustomerCluePoolQueryVO queryVO) {
        final String msg = "查询客户线索池列表[pool/clue/list]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            queryVO.setGroupId(currentUser.getGroupId());
            AppPage<CustomerCluePoolVO> page = poolBizService.clueList(queryVO);
            return success(page, data -> log.info("dataSize: {}", CollectionUtils.isEmpty(data.getData()) ? 0 : data.getData().size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO.getType(), e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    @GetMapping("/clue/summary")
    @IgnoreAuth
    public Message<CustomerClueSummaryVO> clueSummary(@CurrentUser LoginAuthBean currentUser, final CustomerCluePoolQueryVO queryVO) {
        final String msg = "查询客户线索池概况[pool/clue/summary]";
        try {
            log.info("{}: param[{}]", msg, queryVO);
            queryVO.setGroupId(currentUser.getGroupId());
            return success(poolBizService.summary(queryVO), data -> log.info("summary: {}", JSONObject.toJSONString(data)));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO.getType(), e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }
}
