package cn.fw.valhalla.controller.app;

import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.query.StammkundeAnalyseQueryVO;
import cn.fw.valhalla.domain.vo.customer.StammkundeAddressAnalyseVO;
import cn.fw.valhalla.domain.vo.customer.StammkundeAnalyseVO;
import cn.fw.valhalla.service.bus.StammkundeBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.valhalla.common.constant.MessageStr.QUERY_FAILURE;

/**
 * @author : kurisu
 * @className : StammkundeController
 * @description : 保有客分析控制器
 * @date: 2021-01-21 15:28
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@RequiredArgsConstructor
@Validated
@RequestMapping("/app/stammkunde")
public class StammkundeController {
    private final StammkundeBizService stammkundeBizService;

    /**
     * 根据关单任务对保有客地址进行分析
     *
     * @param queryVO
     * @return
     */
    @GetMapping("/address/analyse/task")
    @IgnoreAuth
    public Message<List<StammkundeAddressAnalyseVO>> taskAddressList(@CurrentUser LoginAuthBean currentUser, final StammkundeAnalyseQueryVO queryVO) {
        final String msg = "根据关单任务对保有客地址进行分析[stammkunde/address/analyse/task]";
        try {
            queryVO.setGroupId(currentUser.getGroupId());
            log.info("{}: param[{}]", msg, queryVO);
            return success(stammkundeBizService.taskAddressList(queryVO));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    /**
     * 对保有客地址进行分析
     *
     * @param queryVO
     * @return
     */
    @GetMapping("/address/analyse/customer")
    @IgnoreAuth
    public Message<List<StammkundeAddressAnalyseVO>> stammkundeAddressList(@CurrentUser LoginAuthBean currentUser, final StammkundeAnalyseQueryVO queryVO) {
        final String msg = "对保有客地址进行分析[stammkunde/address/analyse/customer]";
        try {
            queryVO.setGroupId(currentUser.getGroupId());
            log.info("{}: param[{}]", msg, queryVO);
            return success(stammkundeBizService.customerAddressList(queryVO));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }

    /**
     * 查询半径范围内的客户
     *
     * @param queryVO
     * @return
     */
    @GetMapping("/distance/analyse")
    @IgnoreAuth
    public Message<List<StammkundeAnalyseVO>> stammkundeList(@CurrentUser LoginAuthBean currentUser, final StammkundeAnalyseQueryVO queryVO) {
        final String msg = "查询半径范围内的客户[stammkunde/distance/analyse]";
        try {
            queryVO.setGroupId(currentUser.getGroupId());
            log.info("{}: param[{}]", msg, queryVO);
            return success(stammkundeBizService.stammkundeList(queryVO));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, queryVO, e));
            return failureWithMessage(QUERY_FAILURE);
        }
    }
}
