package cn.fw.valhalla.controller.wx;

import cn.fw.common.web.auth.PassportAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.CorrelationDto;
import cn.fw.valhalla.domain.dto.CustomerChangeDto;
import cn.fw.valhalla.domain.vo.customer.CarArchiveVO;
import cn.fw.valhalla.domain.vo.customer.CustomerChangeInfoVO;
import cn.fw.valhalla.service.bus.cust.ContactBizService;
import cn.fw.valhalla.service.bus.cust.CustomerBizService;
import cn.fw.valhalla.service.bus.cust.CustomerChangeBizService;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.valhalla.common.constant.MessageStr.SAVE_FAILURE;

/**
 * @author : kurisu
 * @className : CustomerWxController
 * @description : 档案微信端控制器
 * @date: 2020-08-13 15:00
 */
@Slf4j
@Validated
@RestController
@Authorization(AuthType.WECHAT)
@RequestMapping("/wx/customer")
public class CustomerWxController {
    private final CustomerBizService customerBizService;
    private final CustomerChangeBizService changeBizService;
    private final ContactBizService contactBizService;

    @Autowired
    public CustomerWxController(final CustomerBizService customerBizService,
                                final CustomerChangeBizService changeBizService,
                                final ContactBizService contactBizService) {
        this.customerBizService = customerBizService;
        this.changeBizService = changeBizService;
        this.contactBizService = contactBizService;
    }

    /**
     * C端-查询是否关联保有客档案
     *
     * @param user       当前登录用户
     * @param customerId 保有客档案id
     * @return 操作结果
     */
    @GetMapping("/exist/correlation")
    public Message<Boolean> existCorrelation(@CurrentUser PassportAuthBean user, @NotNull(message = "档案ID不能为空") Long customerId) {
        final String msg = "C端-查询是否关联保有客档案[existCorrelation]";
        log.info("{},param:[{}]", msg, customerId);
        try {
            return success(contactBizService.existCorrelation(user.getUserId(), customerId),
                    data -> log.info("{} param[{},{}] data[{}]", msg, user, customerId, data));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：", msg, e));
            return failureWithMessage("查询是否关联保有客档案失败");
        }
    }

    /**
     * C端wx-档案关联送修人
     *
     * @param user           当前登录用户
     * @param correlationDto 联系人信息
     * @return 操作结果
     */
    @PostMapping("/correlation")
    public Message<Boolean> correlation(@CurrentUser PassportAuthBean user,
                                        @Validated @RequestBody CorrelationDto correlationDto) {
        final String msg = "C端wx-关联送修人[correlation]";
        log.info("{},param:[{}]", msg, JSONObject.toJSONString(correlationDto));
        try {
            return success(contactBizService.correlation(user, correlationDto));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：", msg, e));
            return failureWithMessage("关联送修人失败");
        }
    }

    /**
     * C端wx-档案查询
     *
     * @param cusId
     * @return
     */
    @GetMapping("/info")
    public Message<CustomerChangeInfoVO> info(@NotNull(message = "档案id不能为空") Long cusId) {
        final String msg = "C端wx-保有客档案查询[/info]";
        log.info("{},param:[{}]", msg, cusId);
        try {
            return success(changeBizService.mpGetById(cusId));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：", msg, e));
            return failureWithMessage("保有客档案信息获取失败");
        }
    }

    /**
     * 回显用户修改的基本信息
     *
     * @param user
     * @param key
     * @return
     */
    @GetMapping("/change/info")
    public Message<CustomerChangeInfoVO> changeInfo(@CurrentUser PassportAuthBean user,
                                                    @NotBlank(message = "口令不能为空") String key) {
        final String msg = "C端wx-保有客档案变更基本信息回显[/change/info]";
        log.info("{},param:[{}]", msg, key);
        try {
            return success(changeBizService.getInfo(user, key));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：", msg, e));
            return failureWithMessage("保有客档案变更基本信息获取失败");
        }
    }

    /**
     * 档案详情修改
     *
     * @param dto 档案信息
     */
    @PostMapping("/save")
    public Message<Long> updateOrCreate(@CurrentUser PassportAuthBean user, @Validated @RequestBody final CustomerChangeDto dto) {
        final String msg = "修改档案信息[save]";
        log.info("{}: param[{}]", msg, dto);
        try {
            return success(changeBizService.updateOrSave(user, dto));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：param[{}]", msg, dto, e));
            return failureWithMessage(SAVE_FAILURE);
        }
    }

    /**
     * 查询所有档案
     *
     * @param user
     * @return
     */
    @GetMapping("/car/list")
    public Message<List<CarArchiveVO>> carList(@CurrentUser PassportAuthBean user) {
        final String msg = "C端wx-查询所有档案[/car/default]";
        log.info("{}", msg);
        try {
            return success(customerBizService.defaultCustomer(user.getUserId()),
                    data -> log.info("{}: data.size[{}]", msg, data.size()));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{}失败：", msg, e));
            return failureWithMessage("查询所有档案失败");
        }
    }
}
