package cn.fw.valhalla.service.bus;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.data.base.domain.common.Message;
import cn.fw.third.push.sdk.api.ImSendMessage;
import cn.fw.third.push.sdk.api.para.im.MsgPara;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.enums.AllocationTypeEnum;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.dto.StammkundeDto;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.rpc.erp.TodoRpcService;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.BackLogItemDTO;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : LeaveNeedDoBizService
 * @description : 人员变动处理
 * @date: 2020-10-16 17:04
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class LeaveNeedDoBizService {
    private final LeaveNeedDoService leaveNeedDoService;
    private final CustomerService customerService;
    private final UserService userService;
    private final OopService oopService;
    private final FollowBizService followBizService;
    private final FollowTaskService followTaskService;
    private final DistributedLocker distributedLocker;
    private final ImSendMessage imSendMessage;
    private final TodoRpcService todoRpcService;
    private final CustomerCluePoolService customerCluePoolService;
    private final StammkundePoolService stammkundePoolService;

    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:LeaveNeedDo")
    @Getter
    private String keyPrefix;

    @Value("${follow.todo.leave2do}")
    @Getter
    private String leave2doCode;


    @Transactional(rollbackFor = Exception.class)
    public void add(final Long userId, final Long shopId) {
        String lockKey = String.format("%s:add:lock:%s:%s", getKeyPrefix(), shopId, userId);
        Pair<Boolean, RLock> pair = distributedLocker.tryLock(lockKey, TimeUnit.SECONDS, 0, 30);
        BV.isTrue(Boolean.TRUE.equals(pair.getKey()), () -> "请勿重复提交");
        try {
            List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
            BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
            LeaveNeedDo db = addable(userId, shopId);
            leaveNeedDoService.save(db);
            push2Todo(db.getId(), postUserDTOS.get(0).getUserId(), shopId);
        } catch (Exception e) {
            distributedLocker.unlock(lockKey);
            throw e;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void add(LeaveNeedDo leaveNeedDo) {
        Long userId = leaveNeedDo.getUserId();
        Long shopId = leaveNeedDo.getShopId();
        List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
        boolean bool = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        ) > 0;
        if (bool) {
            return;
        }
        bool = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        ) <= 0;
        if (bool) {
            return;
        }
        leaveNeedDoService.save(leaveNeedDo);
        push2Todo(leaveNeedDo.getId(), postUserDTOS.get(0).getUserId(), shopId);
    }

    public AppPage<LeaveNeedDoVO> getList(LoginAuthBean currentUser, LeaveQueryVO queryVO) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(currentUser.getUserId(), RoleCode.BYKFP);
        List<Long> shopIds = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
        BV.isFalse(CollectionUtils.isEmpty(shopIds), () -> "无权限操作，请检查角色权限是否正确");
        PageData<LeaveNeedDo> pageData = leaveNeedDoService.page(new PageData<>(queryVO), Wrappers.<LeaveNeedDo>lambdaQuery()
                .in(LeaveNeedDo::getShopId, shopIds)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
                .eq(Objects.nonNull(queryVO.getType()), LeaveNeedDo::getType, queryVO.getType())
        );
        AppPage<LeaveNeedDoVO> page = AppPage.empty(queryVO);
        List<LeaveNeedDoVO> list = new ArrayList<>();
        List<LeaveNeedDo> records = Optional.ofNullable(pageData.getRecords()).orElse(new ArrayList<>());
        for (LeaveNeedDo needDo : records) {
            int i = customerService.count(Wrappers.<Customer>lambdaQuery()
                    .eq(Customer::getAdviserId, needDo.getUserId())
                    .eq(Customer::getYn, Boolean.TRUE)
            );
            list.add(LeaveNeedDoVO.with(needDo, i));
        }
        page.setData(list);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    public void allocation(LoginAuthBean user, LeaveAllocationDTO dto) {
        final String key = generateKey(dto.getId());
        final String lockKey = getLockKey(dto.getId());
        Pair<Boolean, RLock> pair = distributedLocker.tryLock(lockKey, TimeUnit.SECONDS, 0, 30);
        BV.isTrue(Boolean.TRUE.equals(pair.getKey()), () -> "请勿重复提交");
        try {
            //准备分配 常规校验
            prepareAllocation(dto);
            //变更档案
            List<Customer> customerList = doAllocation(dto, key);
            //分配跟进任务
            dealFollowTask(customerList);
            //完成分配发送消息提醒等
            finish(user, dto.getId(), key);
        } catch (Exception e) {
            clearKey(key);
            distributedLocker.unlock(lockKey);
            throw e;
        }
    }

    private void prepareAllocation(LeaveAllocationDTO dto) {
        AllocationTypeEnum typeEnum = AllocationTypeEnum.ofValue(dto.getAllocationType());
        BV.notNull(typeEnum, () -> "分配方式不正确,请重试");
        dto.setType(typeEnum);
        LeaveNeedDo needDo = leaveNeedDoService.queryProcessableById(dto.getId());
        BV.notNull(needDo, () -> "该条记录已处理或不存在，请刷新后重试");

        dto.setAdviserId(needDo.getUserId());
        if (AllocationTypeEnum.ONE.equals(typeEnum)) {
            BV.notNull(dto.getUserId(), () -> "指定人员不能为空");
            List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(dto.getUserId(), RoleCode.FWGW);
            List<Long> shopIdList = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
            BV.isNotEmpty(shopIdList, () -> "指定人员非服务顾问角色，请核对");
            Long shopId = shopIdList.get(0);
            BV.isTrue(needDo.getShopId().equals(shopId), () -> "指定人员所属门店与档案归属门店不符");
            dto.setShopId(shopId);
        }

        if (AllocationTypeEnum.ALL.equals(dto.getType())) {
            dto.setShopId(needDo.getShopId());
        }
    }

    /**
     * 处理数据
     *
     * @param dto
     * @return 处理完的档案
     */
    private List<Customer> doAllocation(LeaveAllocationDTO dto, String key) {
        List<Customer> customerList = customerService.queryByAdviserId(dto.getAdviserId());
        if (CollectionUtils.isEmpty(customerList)) {
            return null;
        }
        if (AllocationTypeEnum.ONE.equals(dto.getType())) {
            allocation(key, customerList, dto);
        }
        if (AllocationTypeEnum.ALL.equals(dto.getType())) {
            allocation(key, customerList, dto.getShopId());
        }
        return customerList;
    }

    /**
     * 分配给指定人员
     *
     * @param list
     * @param dto
     */
    private void allocation(String key, List<Customer> list, LeaveAllocationDTO dto) {
        List<StammkundePool> spl = new ArrayList<>(list.size());
        final String userName = Optional.ofNullable(userService.user(dto.getUserId())).map(UserInfoDTO::getUserName).orElse("");
        for (Customer customer : list) {
            customer.setShopId(dto.getShopId());
            customer.setAdviserId(dto.getUserId());
            spl.add(createPool(customer, userName));
            rejectPool(customer.getId(), dto.getShopId(), userName, dto.getUserId(), customer.getGroupId());
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        setToCache(key, new UserInfo(dto.getUserId(), "", list.size()));
    }

    /**
     * 门店内平均分配
     *
     * @param list
     * @param shopId
     */
    private void allocation(String key, List<Customer> list, Long shopId) {
        List<PostUserDTO> users = userService.getUserByRole(shopId, RoleCode.FWGW);
        BV.isNotEmpty(users, () -> "该门店没有服务顾问，请检查配置是否正确");
        List<StammkundePool> spl = new ArrayList<>(list.size());
        LinkedList<UserInfo> queue = new LinkedList<>();
        for (PostUserDTO user : users) {
            queue.offer(new UserInfo(user.getUserId(), user.getUserName()));
        }
        for (Customer customer : list) {
            UserInfo info = queue.poll();
            customer.setShopId(shopId);
            customer.setAdviserId(Objects.requireNonNull(info, "服务顾问信息获取异常，请重试").getUserId());
            info.setCount(info.getCount() + 1);
            rejectPool(customer.getId(), shopId, info.getUserName(), info.getUserId(), customer.getGroupId());
            spl.add(createPool(customer, info.getUserName()));
            queue.offer(info);
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        setToCache(key, queue);
    }

    /**
     * 完成分配的后续处理逻辑
     *
     * @param list
     */
    private void dealFollowTask(List<Customer> list) {
        List<FollowTypeEnum> enumList = Arrays.asList(FollowTypeEnum.FM, FollowTypeEnum.RM);
        List<ClueStatusEnum> statusList = Arrays.asList(ClueStatusEnum.WAITING, ClueStatusEnum.ONGOING);
        if (!CollectionUtils.isEmpty(list)) {
            //处理待办等信息
            for (Customer customer : list) {
                final Long customerId = customer.getId();
                final Long shopId = customer.getShopId();
                final Long adviserId = customer.getAdviserId();
                List<CustomerCluePool> cluePoolList = customerCluePoolService.list(Wrappers.<CustomerCluePool>lambdaQuery()
                        .eq(CustomerCluePool::getRefererId, customerId)
                        .in(CustomerCluePool::getClueStatus, statusList)
                        .eq(CustomerCluePool::getGroupId, customer.getGroupId())
                        .in(CustomerCluePool::getClueType, enumList)
                );

                dealClue(cluePoolList, adviserId, shopId);
            }
        }
    }

    private void dealClue(List<CustomerCluePool> list, Long adviserId, Long shopId) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (CustomerCluePool clue : list) {
            if (ClueStatusEnum.WAITING.equals(clue.getClueStatus())) {
                clue.setOriginalUserId(adviserId);
                Optional.ofNullable(userService.user(adviserId)).ifPresent(user -> clue.setOriginalUserName(user.getUserName()));
                clue.setOriginalShopId(shopId);
                Optional.ofNullable(oopService.shop(shopId)).ifPresent(shop -> clue.setOriginalShopName(shop.getShortName()));
                continue;
            }
            dealTask(clue, adviserId, shopId);
        }

        customerCluePoolService.updateBatchById(list);
    }

    private void dealTask(CustomerCluePool clue, Long adviserId, Long shopId) {
        if (Boolean.FALSE.equals(clue.getRedistribution())) {
            clue.setRedistribution(Boolean.TRUE);
        }
        FollowTask task = followTaskService.queryOngoingTaskByClueId(clue.getId());
        if (Objects.isNull(task)) {
            return;
        }
        if (Boolean.TRUE.equals(task.getRedistribution())) {
            task.setFollowUser(adviserId);
            task.setFollowShop(shopId);
            followTaskService.updateById(task);
            return;
        }
        task.setCloseTime(new Date());
        task.setState(TaskStateEnum.DEFEAT);
        task.setReason(TaskDefeatTypeEnum.D);
        followTaskService.updateById(task);
        FollowTask nTask = new FollowTask();
        nTask.setClueId(clue.getId());
        nTask.setCustomerId(clue.getRefererId());
        nTask.setType(clue.getClueType());
        nTask.setState(TaskStateEnum.ONGOING);
        nTask.setBeginTime(new Date());
        nTask.setRedistribution(Boolean.TRUE);
        nTask.setDeadline(clue.getDeadline());
        nTask.setFollowUser(adviserId);
        nTask.setFollowShop(shopId);
        nTask.setGroupId(clue.getGroupId());
        followTaskService.save(nTask);
        followBizService.startTask(nTask);
    }

    private void finish(LoginAuthBean user, Long leaveId, String key) {
        leaveNeedDoService.dealById(leaveId);
        List<UserInfo> list = getAllFromCache(key);
        for (UserInfo info : list) {
            try {
                String text = String.format("%s通过资源分配给你%s台保有客", user.getUserName(), info.getCount());
                final MsgPara msgPara = MsgPara.getOfflineTxetPara(text, null, "保有客分配",
                        "保有客分配", null, info.getUserId()).build();
                final Message<Integer> msg = imSendMessage.sendMsg(msgPara);
                log.info("给[{}]推送im消息结果:[{}]", info, msg.getResult());
            } catch (Exception e) {
                log.error("给[{}]推送im消息失败：]", info, e);
            }
        }
        clearKey(key);
        BackLogItemDTO backLogItemDTO = new BackLogItemDTO(user.getUserId(), getLeave2doCode(), String.valueOf(leaveId), new Date(), null);
        todoRpcService.complete(backLogItemDTO);
    }

    private void rejectPool(Long customerId, Long shopId, String userName, Long userId, Long groupId) {
        StammkundeDto stammkundeDto = new StammkundeDto();
        stammkundeDto.setCustomerId(customerId);
        stammkundeDto.setReason(DefeatReasonEnum.LC);
        stammkundeDto.setShopId(shopId);
        stammkundeDto.setNewUserId(userId);
        stammkundeDto.setNewUserName(userName);
        ShopDTO shop = oopService.shop(shopId);
        stammkundeDto.setNewShopId(shopId);
        if (Objects.nonNull(shop)) {
            stammkundeDto.setNewShopName(shop.getShortName());
        }
        stammkundeDto.setGroupId(groupId);

        stammkundePoolService.reject(stammkundeDto);
    }

    private StammkundePool createPool(Customer customer, String userName) {
        StammkundePool sp = new StammkundePool();
        sp.setCustomerId(customer.getId());
        sp.setAdviserId(customer.getAdviserId());
        sp.setAdviserName(userName);
        sp.setSources(StammkundeSourcesEnum.SYSTEMATIC);
        sp.setPoolStatus(StammkundeStatusEnum.KUNDE);
        sp.setAktiv(Boolean.TRUE);
        sp.setActivationTime(new Date());
        sp.setShopId(customer.getShopId());
        sp.setGroupId(customer.getGroupId());
        return sp;
    }

    private String getLockKey(final Long leaveId) {
        BV.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:lock:%s", getKeyPrefix(), leaveId);
    }

    private String generateKey(final Long leaveId) {
        Assert.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:allocation:%s", getKeyPrefix(), leaveId);
    }

    public List<UserInfo> getAllFromCache(final String key) {
        try {
            Boolean hasKey = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(hasKey)) {
                return new ArrayList<>();
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            List<String> stringList = ops.range(0, -1);
            if (CollectionUtils.isEmpty(stringList)) {
                return new ArrayList<>();
            }
            List<UserInfo> dtos = stringList.stream().map(str -> JSONObject.parseObject(str, UserInfo.class)).collect(Collectors.toList());
            redisTemplate.delete(key);
            return dtos;
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
            return new ArrayList<>();
        }
    }

    private void setToCache(final String key, final UserInfo userInfo) {
        try {
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String jsonString = JSONObject.toJSONString(userInfo);
            ops.rightPush(jsonString);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void setToCache(final String key, final LinkedList<UserInfo> userInfos) {
        try {
            if (CollectionUtils.isEmpty(userInfos)) {
                return;
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String[] users = userInfos.stream().map(JSONObject::toJSONString).toArray(String[]::new);
            ops.rightPushAll(users);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void clearKey(final String key) {
        try {
            redisTemplate.delete(key);
        } catch (Exception e) {
            log.error("清空缓存信息失败 key[{}]", key, e);
        }
    }

    private LeaveNeedDo addable(final Long userId, final Long shopId) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(userId, RoleCode.FWGW);
        boolean ok = CollectionUtils.isEmpty(dataRange) || !Objects.equals(dataRange.get(0).getRangeValue(), shopId);
        BV.isTrue(ok, () -> "请先移除对对应服务站服务顾问角色后操作");
        int count = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        );
        BV.isTrue(count == 0, () -> "已存在待分配记录，请勿重复添加");
        UserInfoDTO user = userService.user(userId);
        BV.notNull(user, () -> "用户不存在");
        boolean bool = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        ) > 0;
        BV.isTrue(bool, () -> "该顾问没有可用档案无需分配保有客");
        LeaveNeedDo db = new LeaveNeedDo();
        db.setDone(Boolean.FALSE);
        db.setEffectiveTime(DateUtil.getMonthEndDay(new Date()));
        db.setReason(LeaveReasonEnum.OTHER);
        db.setType(LeaveTodoTypeEnum.CUSTOMER);
        db.setShopId(shopId);
        db.setUserId(userId);
        db.setUserName(user.getUserName());
        db.setCreateTime(new Date());
        db.setUpdateTime(new Date());
        return db;
    }

    private void push2Todo(Long id, Long userId, Long shopId) {
        BackLogItemDTO dto = new BackLogItemDTO(userId, getLeave2doCode(), String.valueOf(id), new Date(), shopId);
        todoRpcService.push(dto);
    }

    static class UserInfo {
        private Long userId;
        private String userName;
        private int count;

        public UserInfo() {
        }

        public UserInfo(Long userId, String userName) {
            this.userId = userId;
            this.userName = userName;
        }

        public UserInfo(Long userId, String userName, int count) {
            this.userId = userId;
            this.userName = userName;
            this.count = count;
        }

        public Long getUserId() {
            return userId;
        }

        public void setUserId(Long userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public int getCount() {
            return count;
        }

        public void setCount(int count) {
            this.count = count;
        }

        @Override
        public String toString() {
            return "UserInfo{" +
                    "userId=" + userId +
                    ", userName='" + userName + '\'' +
                    ", count=" + count +
                    '}';
        }
    }
}
